/*
//  GNotifier.m
//  FMenu
//
//  Created by Paul Borokhov on 12/23/06.
//  Copyright 2006, 2007 Paul Borokhov. All rights reserved.
//
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FMenu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#import "GNotifier.h"
#import "FB_URLs.h"

@implementation GNotifier

- (id)init {
	self = [super init];
	if (self) {
		[GrowlApplicationBridge setGrowlDelegate:self];
	}
	return self;
}

- (NSDictionary *) registrationDictionaryForGrowl {
	NSArray *notificationsArray = [NSArray arrayWithObjects:@"New version available", @"Status update", @"New note", @"Profile updated", @"New messages", @"New pokes", @"New wall posts", @"New friend requests", @"Error", nil];
	return [NSDictionary dictionaryWithObjectsAndKeys:notificationsArray, GROWL_NOTIFICATIONS_ALL, notificationsArray, GROWL_NOTIFICATIONS_DEFAULT, nil];
}

- (NSString *) applicationNameForGrowl {
	return @"FMenu";
}

- (void) doNotify:(NSString *)name type:(NSString *)nType note:(NSString *)status {
	[self doNotify:name type:nType note:status targetID:nil];
}

- (void) doNotify:(NSString *)name type:(NSString *)nType note:(NSString *)status targetID:(NSString *)targetID {
	NSString *description = @"";
	int priority = 0;
	BOOL sticky = NO;
	NSString *targetURL = @"";
	GenericID *tID = [GenericID createID:@""];
	if ([nType isEqual:@"Status update"]) {
		description = [NSString stringWithFormat:@"%@ %@", name, status];
		tID = [ProfileID createID:targetID];
	} else if ([nType isEqual:@"New note"]) {
		description = [NSString stringWithFormat:@"%@ wrote a new note", name];
		targetURL = [[ProfileID createID:targetID] notesURLString];
	} else if ([nType isEqual:@"Profile updated"]) {
		NSString *gender;
		if ([status isEqual:@"male"]) {
			gender = @"his";
		} else if ([status isEqual:@"female"]) {
			gender = @"her";
		} else {
			gender = @"their";
		}
		description = [NSString stringWithFormat:@"%@ updated %@ profile", name, gender];
		tID = [ProfileID createID:targetID];
	} else if ([nType isEqual:@"New messages"]) {
		description = status;
		targetURL = FB_INBOX_URL;
	} else if ([nType isEqual:@"New pokes"]) {
		description = status;
		targetURL = FB_POKES_URL;
	} else if ([nType isEqual:@"New wall posts"]) {
		description = @"You have new wall posts";
		if ([[NSUserDefaults standardUserDefaults] integerForKey:@"wallDestination"] == 0) {
			targetURL = FB_PROFILE_WALL_URL;
		} else {
			targetURL = FB_SOLE_WALL_URL;
		}
	} else if ([nType isEqual:@"New friend requests"]) {
		description = @"You have new friend requests";
		targetURL = FB_REQS_URL;
	} else if ([nType isEqual:@"Error"]) {
		description = status;
		priority = 2;
	} else if ([nType isEqual:@"New version available"]) {
		description = status;
		sticky = YES;
	}
	if (![description isEqual:@""]) {
		if ([targetURL isEqual:@""] && ![[tID ID] isEqual:@""]) {
			targetURL = [tID launchURLString];
		}
		NSDictionary *context = [NSDictionary dictionaryWithObjectsAndKeys: targetURL, @"targetURL", nil];
		[GrowlApplicationBridge notifyWithTitle:nType description:description notificationName:nType iconData:nil priority:priority isSticky:sticky clickContext:context];
	}
}

- (void) growlNotificationWasClicked:(id)clickContext {
	clickContext = (NSDictionary *) clickContext;
	NSString *url = nil;
	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"clickContextActive"]) {
		if ((url = [clickContext valueForKey:@"targetURL"])) {
			if (![url isEqual:@""]) {
				[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:url]];
			}
		}
	}
}

- (void) growlNotificationTimedOut:(id)clickContext {
	/// do nothing
}

@end
