//
//  MenuItems.m
//  FMenu
//
//  Created by Paul Borokhov on 1/5/07.
//  Copyright 2007 Paul Borokhov. All rights reserved.
/*
 **
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FMenu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#import "MenuItems.h"

@implementation Controller (MenuItems)

#pragma mark -
#pragma mark Updating UI on main thread
- (void)updateMenuItem:(id)anUpdate {
	if ([anUpdate isKindOfClass:[MenuItemUpdate class]]) {
		//NSLog(@"about to set menu item title to %@ from %@", [anUpdate title], [[anUpdate menuitem] title]);
		[[anUpdate menuitem] setTitle:[anUpdate title]];
	}
}

#pragma mark Menu Items

- (IBAction)goToMessages:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_INBOX_URL]];
}

- (IBAction)goToNotifications:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_NOTIFICATIONS_URL]];
}

- (IBAction)goToComposer:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_COMPOSE_URL]];
}

- (IBAction)goToHome:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_HOME_URL]];
}

- (IBAction)goToWall:(id)sender {
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_PROFILE_WALL_URL]];
}

- (IBAction)goToEvents:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_EVENTS_URL]];
}

- (IBAction)goToAllEvents:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_ALL_EVENTS_URL]];
}

- (IBAction)goToReqs:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_REQS_URL]];
}

- (IBAction)goToPokes:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_POKES_URL]];
}

- (IBAction)goToFriends:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_FRIENDS_URL]];
}

- (IBAction)forceUpdate:(id)sender {
	[self restartTimers]; // do this first
	[self getCounts]; // otherwise we're going to cancel these timers before they run
}

- (IBAction)showQL:(id)sender {
	// dummy method to enable the menuitem
}

- (IBAction)showAbout:(id)sender {
	[NSApp activateIgnoringOtherApps:YES];
	[NSApp orderFrontStandardAboutPanel:sender]; 
}

- (IBAction)showReadMe:(id)sender {
	[NSApp activateIgnoringOtherApps:YES];
	[readMeView readRTFDFromFile:[[NSBundle bundleForClass:[self class]] pathForResource:@"Read Me" ofType:@"rtf"]];
	
	[readMeWindow center];
	[readMeWindow makeKeyAndOrderFront:sender];
}

- (IBAction)showPreferences:(id)sender {
	[NSApp activateIgnoringOtherApps:YES];
	[preferencesWindow makeKeyAndOrderFront:sender];
}

#pragma mark Home submenu items
- (void) createSubmenus {
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showPhotosSub"] && ![photosMenuItem hasSubmenu]) {
		NSMenu *photosSub = [[NSMenu alloc] initWithTitle:@"Photos"];
		[photosSub addItemWithTitle:@"Photos of Me" action:@selector(goToPhotosAbout:) keyEquivalent:@""];
		[photosSub addItemWithTitle:@"My Friends' Photo Albums" action:@selector(goToPhotosOfFriends:) keyEquivalent:@""];
		[photosSub addItemWithTitle:@"My Recently Tagged Friends" action:@selector(goToPhotosRecentlyTagged:) keyEquivalent:@""];
		[photosMenuItem setSubmenu:photosSub];
	} else if (![[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showPhotosSub"] && [photosMenuItem hasSubmenu]) {
		[[photosMenuItem submenu] release];
		[photosMenuItem setSubmenu:nil];
	}
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showSharesSub"] && ![sharesMenuItem hasSubmenu]) {
		NSMenu *sharesSub = [[NSMenu alloc] initWithTitle:@"Shares"];
		[sharesSub addItemWithTitle:@"Friends' Links" action:@selector(goToSharesOfFriends:) keyEquivalent:@""];
		[sharesMenuItem setSubmenu:sharesSub];
	} else if (![[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showSharesSub"] && [sharesMenuItem hasSubmenu]) {
		[[sharesMenuItem submenu] release];
		[sharesMenuItem setSubmenu:nil];
	}
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showNotesSub"] && ![notesMenuItem hasSubmenu]) {
		NSMenu *notesSub = [[NSMenu alloc] initWithTitle:@"Notes"];
		[notesSub addItemWithTitle:@"Notes About Me" action:@selector(goToNotesAbout:) keyEquivalent:@""];
		[notesSub addItemWithTitle:@"My Friends' Notes" action:@selector(goToNotesOfFriends:) keyEquivalent:@""];
		[notesMenuItem setSubmenu:notesSub];
	} else if (![[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showNotesSub"] && [notesMenuItem hasSubmenu]) {
		[[notesMenuItem submenu] release];
		[notesMenuItem setSubmenu:nil];
	}
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showGroupsSub"] && ![groupsMenuItem hasSubmenu]) {
		NSMenu *groupsSub = [[NSMenu alloc] initWithTitle:@"Shares"];
		[groupsSub addItemWithTitle:@"Recently Updated" action:@selector(goToGroupsUpdated:) keyEquivalent:@""];
		[groupsSub addItemWithTitle:@"Groups I Admin" action:@selector(goToGroupsAdmin:) keyEquivalent:@""];
		[groupsSub addItemWithTitle:@"Secret Groups" action:@selector(goToGroupsSecret:) keyEquivalent:@""];
		[groupsSub addItemWithTitle:@"Sponsored Groups" action:@selector(goToGroupsSponsored:) keyEquivalent:@""];
		[groupsMenuItem setSubmenu:groupsSub];
	} else if (![[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showGroupsSub"] && [groupsMenuItem hasSubmenu]) {
		[[groupsMenuItem submenu] release];
		[groupsMenuItem setSubmenu:nil];
	}
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showFriendsSub"] && ![friendsMenuItem hasSubmenu]) {
		NSMenu *friendSub = [[NSMenu alloc] initWithTitle:@"Friends"];
		[friendSub addItemWithTitle:@"Recently Updated" action:@selector(goToFriendsUpdated:) keyEquivalent:@""];
		[friendSub addItemWithTitle:@"Recently Added" action:@selector(goToFriendsAdded:) keyEquivalent:@""];
		//[friendSub addItemWithTitle:@"Online Now" action:@selector(goToFriendsOnline:) keyEquivalent:@""];
		[friendSub addItemWithTitle:@"Status Updates" action:@selector(goToFriendsStatus:) keyEquivalent:@""];
		//[friendSub addItemWithTitle:@"Social Timeline" action:@selector(goToFriendsTimeline:) keyEquivalent:@""];
		//[friendSub addItemWithTitle:@"Friend Game" action:@selector(goToFriendsGame:) keyEquivalent:@""];
		[friendsMenuItem setSubmenu:friendSub];
	} else if (![[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showFriendsSub"] && [friendsMenuItem hasSubmenu]) {
		[[friendsMenuItem submenu] release];
		[friendsMenuItem setSubmenu:nil];
	}
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showMessagesSub"] && ![unreadMessagesMenuItem hasSubmenu]) {
		NSMenu *messagesSub = [[NSMenu alloc] initWithTitle:@"Inbox"];
		[messagesSub addItemWithTitle:@"Sent Messages" action:@selector(goToSharesSent:) keyEquivalent:@""];
		[messagesSub addItemWithTitle:@"Compose New Message" action:@selector(goToComposer:) keyEquivalent:@""];
		[messagesSub addItemWithTitle:@"Updates" action:@selector(goToUpdates:) keyEquivalent:@""];
		[messagesSub addItemWithTitle:@"Notifications" action:@selector(goToNotifications:) keyEquivalent:@""];
		[unreadMessagesMenuItem setSubmenu:messagesSub];
	} else if (![[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"showMessagesSub"] && [unreadMessagesMenuItem hasSubmenu]) {
		[[unreadMessagesMenuItem submenu] release];
		[unreadMessagesMenuItem setSubmenu:nil];
	}
}

- (IBAction)goToProfile:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_PROFILE_URL]];
}
- (IBAction)goToSharesSent:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_SENT_MSGS_URL]];
}
- (IBAction)goToUpdates:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_UPDATES_URL]];
}
- (IBAction)goToFriendsUpdated:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_FRIENDS_RECENTLY_UPDATED_URL]];
}
- (IBAction)goToFriendsAdded:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_FRIENDS_RECENTLY_ADDED_URL]];
}
- (IBAction)goToFriendsStatus:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_STATUS_UPDATES_URL]];
}
- (IBAction)goToPhotos:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:FB_PHOTOS_BY_URL, [facebook uid]]]];
}
- (IBAction)goToPhotosAbout:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:FB_PHOTOS_OF_URL, [facebook uid]]]];
}
- (IBAction)goToPhotosOfFriends:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_PHOTOS_MAIN_URL]]; // ?f
}
- (IBAction)goToPhotosRecentlyTagged:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_PHOTOS_RECENTTAGS_URL]];
}
- (IBAction)goToSharesPosted:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:FB_LINKS_BY_URL, [facebook uid]]]];
}
- (IBAction)goToSharesOfFriends:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_LINKS_MAIN_URL]];
}
- (IBAction)goToNotes:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:FB_NOTES_BY_URL, [facebook uid]]]];
}
- (IBAction)goToNotesAbout:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:FB_NOTES_OF_URL, [facebook uid]]]];
}
- (IBAction)goToNotesOfFriends:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_NOTES_BY_FRIENDS_URL]];
}
- (IBAction)goToGroups:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://www.facebook.com/grouphome.php?id=%@&gv=2", [facebook uid]]]];
}
- (IBAction)goToGroupsUpdated:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://www.facebook.com/grouphome.php?id=%@&gv=1", [facebook uid]]]];
}
- (IBAction)goToGroupsAdmin:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://www.facebook.com/grouphome.php?id=%@&gv=12", [facebook uid]]]];
}
- (IBAction)goToGroupsSecret:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://www.facebook.com/grouphome.php?id=%@&gv=10", [facebook uid]]]];
}
- (IBAction)goToGroupsSponsored:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://www.facebook.com/grouphome.php?id=%@&gv=13", [facebook uid]]]];
}
- (IBAction)goToMobile:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_MOBILE_URL]];
}
- (IBAction)goToAccount:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_ACCOUNT_URL]];
}
- (IBAction)goToPrivacy:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:FB_PRIVACY_URL]];
}

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem {
	if ([facebook userLoggedIn]) {
		if (sleeping && [menuItem action] == @selector(forceUpdate:)) {
			return NO;
		} else {
			return YES;
		}
	} else {
		if ([menuItem action] == @selector(reset:) || [menuItem action] == @selector(showAbout:) || [menuItem action] == @selector(showReadMe:) || [menuItem action] == @selector(showQL:) || [menuItem action] == @selector(showPreferences:)) {
			return YES;
		} else {
			return NO;
		}
	}
}

@end
