//
//  UpdateMethods.m
//  FMenu
//
//  Created by Paul Borokhov on 1/5/07.
//  Copyright 2007 Paul Borokhov. All rights reserved.
/*
 **
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * FMenu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#import "UpdateMethods.h"

@implementation Controller (UpdateMethods)

#pragma mark -
#pragma mark Perform Update

- (void) getCounts {
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) {
		NSLog(@"Performing full update");
	}
	@try {
		for (whoseTurn; whoseTurn<=noOfChecks; whoseTurn++) {
			[NSTimer scheduledTimerWithTimeInterval:firstSleepInterval*whoseTurn target:self selector:@selector(getCountsWithTimer:) userInfo:nil repeats:NO];
		}
	} @catch (NSException * e) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog(@"%@", [e reason]);
		}
	}
}

- (void) getCountsWithTimer:(NSTimer *)aTimer {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	if ([facebook uid] == nil) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog(@"no uid");
		}
		[self doLogout:NO];
		[self doLogin];
	} else {
		if (sleeping) {
			sleeping = NO;
			[self setMenuImage:self];
			[forceUpdateMenuItem setTitle:@"Force update now"];
			[forceUpdateMenuItem setEnabled:YES];
		}
		//if (![self checkAvailability]) {
		//} else {
			@try {
				switch (whoseTurn) {
					case 0:
						[self incrementTurn];
						[NSThread detachNewThreadSelector:@selector(getNotifications:) toTarget:self withObject:nil];
						if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) {
							NSLog(@"getting notifications");
						}
							break;
					case 1:
						[self incrementTurn];
						[NSThread detachNewThreadSelector:@selector(getAllEvents:) toTarget:self withObject:nil];
						if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) {
							NSLog(@"getting all events");
						}
							break;
					case 2:
						[self incrementTurn];
						[NSThread detachNewThreadSelector:@selector(getUpcomingEvents:) toTarget:self withObject:nil];
						if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) {
							NSLog(@"getting upcoming events");
						}
							break;
					case 3:
						[self incrementTurn];
						[NSThread detachNewThreadSelector:@selector(getFriends:) toTarget:self withObject:nil];
						if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) {
							NSLog(@"getting friend profiles");
						}
							break;
					case 4:
						[self incrementTurn];
						[NSThread detachNewThreadSelector:@selector(getGroups:) toTarget:self withObject:nil];
						if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) {
							NSLog(@"getting groups");
						}
							break;
					default:
						[self incrementTurn];
						[NSThread detachNewThreadSelector:@selector(getWallCount:) toTarget:self withObject:nil];
						if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDiagnostics"]) {
							NSLog(@"getting wall");
						}
							break;
				}
			} @catch (NSException * e) {
				[growl doNotify:nil type:@"Error" note:[e reason]];
				if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
					NSLog(@"%@", [e reason]);
				}
			}
			[self calculateCount:self];
		//}
	}
	[pool release];
}

-(void) getProfile:(id)anObject {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	Downloader *profileDownloader = nil;
	NSXMLDocument *profileDocument = nil;
	@try {
		NSURLRequest *request = [NSURLRequest requestWithURL:[facebook facebookUsersGetInfoURL:[NSArray arrayWithObject:[facebook uid]] fields:[NSArray arrayWithObject:@"name"]] cachePolicy:NSURLRequestReloadIgnoringCacheData timeoutInterval:[[NSUserDefaults standardUserDefaults] integerForKey:@"updateTime"]*(noOfChecks-1)];
		
		profileDownloader = [[Downloader alloc] initWithRequest:request];
		if ([profileDownloader hasError] == NO) {
			profileDocument = [[NSXMLDocument alloc] initWithData:[profileDownloader data] options:0 error:nil];
			NSString *userName = [[[profileDocument nodesForXPath:@"./users_getInfo_response/user/name" error:nil] objectAtIndex:0] stringValue];
			
			[self performSelectorOnMainThread:@selector(updateMenuItem:) withObject:[MenuItemUpdate newUpate:usersNameMenuItem withTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"%@'s home", nil, nil), userName]] waitUntilDone:NO];
		}
	} @catch(NSException *exception) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog( @"Fetch Profile Error");
		}
	} @finally {
		[profileDocument release];
		[profileDownloader release];
		[pool release];
	}
}

-(void) getNotifications:(id)anObject {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	Downloader *notificationsDownloader = nil;
	NSXMLDocument *notificationsDocument = nil;
	@try {
		NSURLRequest *request = [NSURLRequest requestWithURL:[facebook facebookNotificationsGetURL] cachePolicy:NSURLRequestReloadIgnoringCacheData timeoutInterval:[[NSUserDefaults standardUserDefaults] integerForKey:@"updateTime"]*(noOfChecks-1)];
		
		notificationsDownloader = [[Downloader alloc] initWithRequest:request];
		if (notificationsDownloader != nil && [notificationsDownloader hasError] == NO) {
			notificationsDocument = [[NSXMLDocument alloc] initWithData:[notificationsDownloader data] options:0 error:nil];
			if ([notificationsDocument rootElement] != nil) {
				if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logXMLData"]) {
					NSLog(@"%@", [[[NSString alloc] initWithData:[notificationsDocument XMLData] encoding:NSUTF8StringEncoding] autorelease]);
				}
				// Messages
				int oldMCount = unreadMessages;
				NSString *unreadMessagesCount = [[[notificationsDocument nodesForXPath:@"./notifications_get_response/messages/unread" error:nil] objectAtIndex:0] stringValue];
				NSString *newMID = [[[notificationsDocument nodesForXPath:@"./notifications_get_response/messages/most_recent" error:nil] objectAtIndex:0] stringValue];
				int difference;
				if (newMID != nil) {
					[[NSScanner scannerWithString:unreadMessagesCount] scanInt:&unreadMessages];
					[self performSelectorOnMainThread:@selector(updateMenuItem:) withObject:[MenuItemUpdate newUpate:unreadMessagesMenuItem withTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"%i Unread Messages", nil, nil), unreadMessages]] waitUntilDone:NO];
					BOOL copyMessages = YES;
					difference = unreadMessages-oldMCount;
					if (difference == 1) {
						[growl doNotify:nil type:@"New messages" note:@"You have a new message"];
					} else if (difference > 1) {
						[growl doNotify:nil type:@"New messages" note:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"You have %i new messages", nil, nil), unreadMessages]];
					} else if (unreadMessages != 0 && (![newMID isEqual:messagesLastID] && ![messagesLastID isEqual:@""])) {
						[growl doNotify:nil type:@"New messages" note:@"You have a new message"];
					} else {
						copyMessages = NO;
					}
					if (copyMessages) {
						[newMID retain];
						[messagesLastID release];
						messagesLastID = newMID;
					}
					
				}
				
				// Pokes
				int oldPCount = unreadPokes;
				NSString *unreadPokesCount = [[[notificationsDocument nodesForXPath:@"./notifications_get_response/pokes/unread" error:nil] objectAtIndex:0] stringValue];
				NSString *newPID = [[[notificationsDocument nodesForXPath:@"./notifications_get_response/pokes/most_recent" error:nil] objectAtIndex:0] stringValue];
				if (newPID != nil) {
					[[NSScanner scannerWithString:unreadPokesCount] scanInt:&unreadPokes];
					[self performSelectorOnMainThread:@selector(updateMenuItem:) withObject:[MenuItemUpdate newUpate:unseenPokesMenuItem withTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"%i Unseen Pokes", nil, nil), unreadPokes]] waitUntilDone:NO];
					BOOL copyPokes = YES;
					difference = unreadPokes-oldPCount;
					if (difference == 1) {
						[growl doNotify:nil type:@"New pokes" note:@"You have an unseen poke"];
					} else if (difference > 1) {
						[growl doNotify:nil type:@"New pokes" note:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"You have %i unseen pokes", nil, nil), unreadPokes]];
					} else if (unreadPokes != 0 && (![newPID isEqual:pokesLastID] && ![pokesLastID isEqual:@""])) {
						[growl doNotify:nil type:@"New pokes" note:@"You have an unseen poke"];
					} else {
						copyPokes = NO;
					}			
					if (copyPokes) {
						[newPID retain];
						[pokesLastID release];
						pokesLastID = newPID;
					}
				}
				
				// Friend requests
				int oldFCount = friendRequestsCount;
				friendRequestsCount = [[[notificationsDocument nodesForXPath:@"./notifications_get_response/friend_requests" error:nil] objectAtIndex:0] childCount];
				[self performSelectorOnMainThread:@selector(updateMenuItem:) withObject:[MenuItemUpdate newUpate:friendRequestsMenuItem withTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"%i Friend Requests", nil, nil), friendRequestsCount]] waitUntilDone:NO];
				if (oldFCount < friendRequestsCount) {
					[growl doNotify:nil type:@"New friend requests" note:nil];
				}				
			}
		}
	} @catch (NSException *exception) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog( @"Notifications Parse Error");
		}
		[self sleep:exception];
	} @finally {
		[notificationsDocument release];
		[notificationsDownloader release];
		[pool release];
	}
}

-(void) getWallCount:(id)anObject {
	// Wall
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	Downloader *wallDownloader = nil;
	NSXMLDocument *wallDocument = nil;
	@try {
		NSURLRequest *request = [NSURLRequest requestWithURL:[facebook facebookUsersGetInfoURL:[NSArray arrayWithObject:[facebook uid]] fields:[NSArray arrayWithObject:@"wall_count"]] cachePolicy:NSURLRequestReloadIgnoringCacheData timeoutInterval:[[NSUserDefaults standardUserDefaults] integerForKey:@"updateTime"]*(noOfChecks-1)];
		
		wallDownloader = [[Downloader alloc] initWithRequest:request];
		if (wallDownloader != nil && [wallDownloader hasError] == NO) {
			wallDocument = [[NSXMLDocument alloc] initWithData:[wallDownloader data] options:0 error:nil];
			int oldCount = wallPosts;
			if ([wallDocument rootElement] != nil) {
				NSString *wallPostsCount = [[[wallDocument nodesForXPath:@"./users_getInfo_response/user/wall_count" error:nil] objectAtIndex:0] stringValue];
				if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logXMLData"]) {
					NSLog(@"%@", [[[NSString alloc] initWithData:[wallDocument XMLData] encoding:NSUTF8StringEncoding] autorelease]);
				}
				if (wallPostsCount != nil) {
					[[NSScanner scannerWithString:wallPostsCount] scanInt:&wallPosts];
					
					[self performSelectorOnMainThread:@selector(updateMenuItem:) withObject:[MenuItemUpdate newUpate:wallPostsMenuItem withTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"%@ Wall Posts", nil, nil), wallPostsCount]] waitUntilDone:NO];
					//[wallPostsMenuItem setTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"%@ Wall Posts", nil, nil), wallPostsCount]];
					int difference = wallPosts-oldCount;
					if (difference > 0 && oldCount != 0) {
						[growl doNotify:nil type:@"New wall posts" note:nil];
					}
				}
			}
		}
	} @catch(NSException *exception) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog( @"Parse Wall Error");
		}
		[self sleep:exception];
	} @finally {
		[wallDocument release];
		[wallDownloader release];
		[pool release];
	}
}

-(void) getAllEvents:(id)anObject {
	// Events
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	Downloader *eventsDownloader = nil;
	NSXMLDocument *eventsDocument = nil;
	@try {
		NSString *fqlQuery = [NSString stringWithFormat:@"select eid, name from event where eid in (select eid from event_member where uid = %@)", [facebook uid]];
		NSURLRequest *request = [NSURLRequest requestWithURL:[facebook facebookFQLQueryURL:fqlQuery] cachePolicy:NSURLRequestReloadIgnoringCacheData timeoutInterval:[[NSUserDefaults standardUserDefaults] integerForKey:@"updateTime"]*(noOfChecks-1)];

		eventsDownloader = [[Downloader alloc] initWithRequest:request];
		if (eventsDownloader != nil && [eventsDownloader hasError] == NO) {
			eventsDocument = [[NSXMLDocument alloc] initWithData:[eventsDownloader data] options:0 error:nil];
			if ([eventsDocument rootElement] != nil) {
				[self performSelectorOnMainThread:@selector(updateMenuItem:) withObject:[MenuItemUpdate newUpate:totalEventsMenuItem withTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"%i Total Events", nil, nil), [[eventsDocument rootElement] childCount]]] waitUntilDone:NO]; //[totalEventsMenuItem setTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"%i Total Events", nil, nil), [[eventsDocument rootElement] childCount]]];
				if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logXMLData"]) {
					NSLog(@"%@", [[[NSString alloc] initWithData:[eventsDocument XMLData] encoding:NSUTF8StringEncoding] autorelease]);
				}
			}
		}	
	} @catch(NSException *exception) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog( @"Parse All Events Error");
		}
		[self sleep:exception];
	} @finally {
		[eventsDocument release];
		[eventsDownloader release];
		[pool release];
	}
}

-(void) getUpcomingEvents:(id)anObject {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	Downloader *eventsDownloader = nil;
	NSXMLDocument *eventsDocument = nil;
	@try {
		NSString *fqlQuery = [NSString stringWithFormat:@"select eid, name from event where eid in (select eid from event_member where uid = %@ and rsvp_status=\"attending\") and start_time >= %i", [facebook uid], (int)[[NSDate date] timeIntervalSince1970]];
		NSURLRequest *request = [NSURLRequest requestWithURL:[facebook facebookFQLQueryURL:fqlQuery] cachePolicy:NSURLRequestReloadIgnoringCacheData timeoutInterval:[[NSUserDefaults standardUserDefaults] integerForKey:@"updateTime"]*(noOfChecks-1)];
		
		eventsDownloader = [[Downloader alloc] initWithRequest:request];
		if (eventsDownloader != nil && [eventsDownloader hasError] == NO) {
			eventsDocument = [[NSXMLDocument alloc] initWithData:[eventsDownloader data] options:0 error:nil];
			if ([eventsDocument rootElement] != nil) {
				if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logXMLData"]) {
					NSLog(@"%@", [[[NSString alloc] initWithData:[eventsDocument XMLData] encoding:NSUTF8StringEncoding] autorelease]);
				}
				eventCount = [[eventsDocument rootElement] childCount];
				[self performSelectorOnMainThread:@selector(updateMenuItem:) withObject:[MenuItemUpdate newUpate:upcomingEventsMenuItem withTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"%i Upcoming Events", nil, nil), eventCount]] waitUntilDone:NO];
				[self fillEvents:[eventsDocument nodesForXPath:@"./fql_query_response/event" error:nil]];
			}
		}
	} @catch(NSException *exception) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog( @"Parse Upcoming Events Error");
		}
		[self sleep:exception];
	} @finally {
		[eventsDocument release];
		[eventsDownloader release];
		[pool release];
	}
}

- (void) fillEvents:(NSArray *)eventsData {
	@synchronized(self) {
		[eventsArray removeAllObjects];
		[eventsDic removeAllObjects];
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"fillEvents"]) {
			for (int i=0; i<[eventsData count]; i++) {
				NSString *name = [[[[eventsData objectAtIndex:i] nodesForXPath:@"./name" error:nil] objectAtIndex:0] stringValue];
				[eventsArray addObject:name];
				[eventsDic setObject:[EventID createID:[[[[eventsData objectAtIndex:i] nodesForXPath:@"./eid" error:nil] objectAtIndex:0] stringValue]] forKey:name];
			}
			[self fillsearch:self];
		}
	}
}

-(void) getFriends:(id)anObject {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	Downloader *friendsDownloader = nil;
	NSXMLDocument *friendsData = nil;
	BOOL copyData = NO;
	@try {
		NSString *fqlQuery = [NSString stringWithFormat:@"select %@ from user where uid in (select uid2 from friend where uid1 = %@)", [fields componentsJoinedByString:@", "], [facebook uid]];
		NSURLRequest *request = [NSURLRequest requestWithURL:[facebook facebookFQLQueryURL:fqlQuery] cachePolicy:NSURLRequestReloadIgnoringCacheData timeoutInterval:[[NSUserDefaults standardUserDefaults] integerForKey:@"updateTime"]*(noOfChecks-1)];
		
		friendsDownloader = [[Downloader alloc] initWithRequest:request];
		if (friendsDownloader != nil && [friendsDownloader hasError] == NO) {
			friendsData = [[NSXMLDocument alloc] initWithData:[friendsDownloader data] options:0 error:nil];
			int i;
			if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logXMLData"]) {
				NSLog(@"%@", [[[NSString alloc] initWithData:[friendsData XMLData] encoding:NSUTF8StringEncoding] autorelease]);
			}
			if ([oldData rootElement] != nil) {
				if ([friendsData rootElement] != nil) {
					copyData = YES;
					NSArray *oldFriends = [oldData nodesForXPath:@"./fql_query_response/user" error:nil];
					NSArray *newFriends = [friendsData nodesForXPath:@"./fql_query_response/user" error:nil];
					int friendsCount = [newFriends count];
					NSMutableDictionary *newNotesDic = [NSMutableDictionary dictionaryWithCapacity:friendsCount];
					NSMutableDictionary *newStatusDic = [NSMutableDictionary dictionaryWithCapacity:friendsCount];
					NSMutableDictionary *newDateDic = [NSMutableDictionary dictionaryWithCapacity:friendsCount];
					
					[self fillFriendsBox:[friendsData nodesForXPath:@"./fql_query_response/user" error:nil]];
					for (i = 0; i<[newFriends count]; i++) {
						NSString *ID = [[[[newFriends objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:nameIndex-1]] error:nil] objectAtIndex:0] stringValue];
						if (ID != @"") {
							// case 1: notes
							[newNotesDic setObject:[[[[newFriends objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:startIndex]] error:nil] objectAtIndex:0] stringValue] forKey:ID];
							// case 2: status
							[newStatusDic setObject:[[[newFriends objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:startIndex+1]] error:nil] objectAtIndex:0] forKey:ID];
							// case 3: profile update time
							[newDateDic setObject:[[[[newFriends objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:startIndex+3]] error:nil] objectAtIndex:0] objectValue] forKey:ID];
						}
					}
					for (i = 0; i<[oldFriends count]; i++) {
						NSString *ID = [[[[oldFriends objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:nameIndex-1]] error:nil] objectAtIndex:0] stringValue];
						NSString *name = [[[[oldFriends objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:nameIndex]] error:nil] objectAtIndex:0] stringValue];
						if (ID != @"") {
							NSString *notesCount = [[[[oldFriends objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:startIndex]] error:nil] objectAtIndex:0] stringValue];
							if (notesCount != @"" && ![notesCount isEqual:[newNotesDic objectForKey:ID]] && ![[newNotesDic objectForKey:ID] isEqual:@""]) {
								[growl doNotify:name type:@"New note" note:nil targetID:ID];
							}
							NSXMLNode *statusNode = [[[oldFriends objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:startIndex+1]] error:nil] objectAtIndex:0];
							NSString *oldStatusTime = [[statusNode childAtIndex:1] stringValue];
							NSString *newStatusTime = [[[newStatusDic objectForKey:ID] childAtIndex:1] stringValue];
							int oldTime, newTime;
							if (oldStatusTime != nil && newStatusTime != nil) {
								[[NSScanner scannerWithString:oldStatusTime] scanInt:&oldTime];
								[[NSScanner scannerWithString:newStatusTime] scanInt:&newTime];
								if (![statusNode isEqual:[newStatusDic objectForKey:ID]]) {
									if (![[[[newStatusDic objectForKey:ID] childAtIndex:0] objectValue] isEqual:@""] &&
										[[newStatusDic objectForKey:ID] childCount] > 0 &&
										oldTime <= newTime) {
										[growl doNotify:name type:@"Status update" note:[[[newStatusDic objectForKey:ID] childAtIndex:0] objectValue] targetID:ID];
									}
								}
								NSString *updateTime = [[[[oldFriends objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:startIndex+3]] error:nil] objectAtIndex:0] objectValue];
								if (updateTime != @"" && ![updateTime isEqual:[newDateDic objectForKey:ID]] && ![[newDateDic objectForKey:ID] isEqual:@""]) {
									[growl doNotify:name type:@"Profile updated" note:[[[[oldFriends objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:startIndex+4]] error:nil] objectAtIndex:0] stringValue] targetID:ID];
								}
							}
						}
					}
				}
			} else {
				copyData = YES;
				[self fillFriendsBox:[friendsData nodesForXPath:@"./fql_query_response/user" error:nil]];
			}
		}
	} @catch(NSException *exception) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog(@"Parse Friend Profiles Error");
		}
		[self sleep:exception];
	} @finally {
		// copy current friendsData to oldData for the next run
		if (copyData) {
			// friendsData retained implicitly since were created using [[... alloc] init]
			[oldData release];
			oldData = friendsData;
		} else {
			[friendsData release];
		}
		[friendsDownloader release];
		[pool release];
	}
}

- (void) fillFriendsBox:(NSArray *)friendsData {
	@synchronized(self) {
		[namesArray removeAllObjects];
		[namesDic removeAllObjects];
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"fillFriends"]) {
			for (int i=0; i<[friendsData count]; i++) {
				NSString *name = [[[[friendsData objectAtIndex:i] nodesForXPath:[@"./" stringByAppendingString:[fields objectAtIndex:nameIndex]] error:nil] objectAtIndex:0] stringValue];
				[namesArray addObject:name];
				[namesDic setObject:[ProfileID createID:[[[[friendsData objectAtIndex:i] nodesForXPath:@"./uid" error:nil] objectAtIndex:0] stringValue]] forKey:name];
			}
			[self performSelectorOnMainThread:@selector(updateMenuItem:) withObject:[MenuItemUpdate newUpate:friendsMenuItem withTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"%i Total Friends available", nil, nil), [friendsData count]]] waitUntilDone:NO];
			[self fillsearch:self];
		}
	}
}

-(void) getGroups:(id)anObject {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	Downloader *groupsDownloader = nil;
	NSXMLDocument *groupsDocument = nil;
	@try {
		NSString *fqlQuery = [NSString stringWithFormat:@"select gid, name from group where gid in (select gid from group_member where uid = %@)", [facebook uid]];
		NSURLRequest *request = [NSURLRequest requestWithURL:[facebook facebookFQLQueryURL:fqlQuery] cachePolicy:NSURLRequestReloadIgnoringCacheData timeoutInterval:[[NSUserDefaults standardUserDefaults] integerForKey:@"updateTime"]*(noOfChecks-1)];

		groupsDownloader = [[Downloader alloc] initWithRequest:request];
		if (groupsDownloader != nil && [groupsDownloader hasError] == NO) {
			groupsDocument = [[NSXMLDocument alloc] initWithData:[groupsDownloader data] options:0 error:nil];
			if ([groupsDocument rootElement] != nil) {
				int groupsCount = [[[groupsDocument nodesForXPath:@"./fql_query_response" error:nil] objectAtIndex:0] childCount];
				if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logXMLData"]) {
					NSLog(@"%@", [[[NSString alloc] initWithData:[groupsDocument XMLData] encoding:NSUTF8StringEncoding] autorelease]);
				}
				
				[self performSelectorOnMainThread:@selector(updateMenuItem:) withObject:[MenuItemUpdate newUpate:groupsMenuItem withTitle:[NSString localizedStringWithFormat:NSLocalizedStringFromTable(@"My Groups (%i)", nil, nil), groupsCount]] waitUntilDone:NO];
				[self fillGroups:[groupsDocument nodesForXPath:@"./fql_query_response/group" error:nil]];
			}
		}
	} @catch(NSException *exception) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog(@"Parse groups error");
		}
		[self sleep:exception];
	} @finally {
		[groupsDocument release];
		[groupsDownloader release];
		[pool release];
	}
}

- (void) fillGroups:(NSArray *)groupsData {
	@synchronized(self) {
		[groupsArray removeAllObjects];
		[groupsDic removeAllObjects];
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"fillGroups"]) {
			for (int i=0; i<[groupsData count]; i++) {
				NSString *name = [[[[groupsData objectAtIndex:i] nodesForXPath:@"./name" error:nil] objectAtIndex:0] stringValue];
				[groupsArray addObject:name];
				[groupsDic setObject:[GroupID createID:[[[[groupsData objectAtIndex:i] nodesForXPath:@"./gid" error:nil] objectAtIndex:0] stringValue]] forKey:name];
			}
			[self fillsearch:self];
		}
	}
}

- (IBAction) fillsearch:(id)sender {
	//@synchronized(self) {
    if (quicklaunchArray && idDic && namesDic && eventsDic && groupsDic && nameSearch) {
		if (quicklaunchArray) [quicklaunchArray removeAllObjects];
		if (idDic) [idDic removeAllObjects];
		if (quicklaunchArray && namesArray) [quicklaunchArray addObjectsFromArray:namesArray];
		if (idDic && namesDic) [idDic addEntriesFromDictionary:namesDic];
		if (quicklaunchArray && eventsArray) [quicklaunchArray addObjectsFromArray:eventsArray];
		if (idDic && eventsDic) [idDic addEntriesFromDictionary:eventsDic];
		if (quicklaunchArray && groupsArray) [quicklaunchArray addObjectsFromArray:groupsArray];
		if (idDic && groupsDic) [idDic addEntriesFromDictionary:groupsDic];
		if (quicklaunchArray) [quicklaunchArray sortUsingSelector:@selector(caseInsensitiveCompare:)];
		if (nameSearch) [nameSearch reloadData];
    }
	//}
}

- (IBAction) calculateCount:(id)sender {
	totalUnread = 0;
	// Messages
	if([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"countUnreadMessages"]) {
		totalUnread += unreadMessages;
	}
	
	// Pokes
	if([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"countUnseenPokes"]) {
		totalUnread += unreadPokes;
	}
	
	// Events
	if([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"countUpcomingEvents"]) {
		totalUnread += eventCount;
	}
	
	// Friend Requests
	if([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"countFriendRequests"]) {
		totalUnread += friendRequestsCount;
	}
	
	// Wall posts
	if([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"countWallPosts"]) {
		totalUnread += wallPosts;
	}
	
	[self updateCount:totalUnread];	
}

-(void) updateCount:(int)count {
    if (totalUnread == 0 && ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"dontDisplay0"] || ![facebook userLoggedIn])) {
        [statusItem setTitle:@""];
    } else {
        [statusItem setTitle:[[NSNumber numberWithInt:totalUnread] stringValue]];
    }
}

-(void) sleep:(NSException *)exception {
	if (exception != nil) {
		if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
			NSLog(@"%@ %@", [exception name], [exception reason]);
		}
	}
	/*[timer invalidate]; // not necessary i think, considering what we do next
	[[NSRunLoop currentRunLoop] cancelPerformSelectorsWithTarget:self];
	sleeping = YES;
	[self startTimer:errorSleepTime];
	[statusItem setImage:[NSImage imageNamed:sleepingImageName]];
	[forceUpdateMenuItem setTitle:@"Sleeping due to errors"];
	[forceUpdateMenuItem setEnabled:NO]; */
}

- (void) incrementTurn {
	if (whoseTurn < (noOfChecks-1)) {
		whoseTurn++;
	} else {
		whoseTurn = 0;
	}
}

@end
